/*
 * id3Attr
 *
 * id3Attr.cpp - main file
 * (c) 1999 Ari Pernick (pernick@uiuc.edu)
 * Based very partially on ID3ren (by Robert Alto (badcrc@tscnet.com)
 * and its BeOS port by Shayne White (shayne@curvedspace.org))
 * Which was based on (c) 1999 Pieter Panman (ppanman@dds.nl)
 * 
 * This program is free software you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
 * USA.
 */

#include "id3Attr.h"
#include "genre.h"
#include <string.h>
#include <stdlib.h>
#include <Path.h>
#include <File.h>
#include <stdio.h>
#include <NodeInfo.h>
#include <Node.h>
#include <Directory.h>
#include <InterfaceDefs.h>
#include <kernel/fs_info.h>
#include <kernel/fs_index.h>
#include <errno.h>


bool recursive = false;

void RetrieveFromFile(BFile* myFile, BEntry* entry);

bool ValidMp3(BEntry *entry)
{
	BNode target(entry);
	BNodeInfo info(&target);
	char buf[B_ATTR_NAME_LENGTH];
	
	info.GetType(buf);

	if ( strncmp (buf, "audio/x-mpeg", 13) == 0 ) 
		return true;
	else
		return false; 
}


bool OpenFile(BEntry* entry, BFile* myFile)
{
	// Recurse the directories
	if (recursive && entry->IsDirectory()) 
	{

		BDirectory dir(entry);
		BEntry entryItr;
			
		dir.Rewind();
		while (dir.GetNextEntry(&entryItr, false) == B_OK) 
		{
			BFile fileItr;
			
			if (OpenFile(&entryItr,&fileItr)) 
				RetrieveFromFile(&fileItr, &entryItr);
		}
		return false;
	}	
	
	
	// if it is not an audio/x-mpeg file, return with error
	if (!ValidMp3(entry))
	{
		char name[B_FILE_NAME_LENGTH];
		
		entry->GetName(name);
		printf("%s is not an audio/x-mpeg file\n",name);
		return false;
	}

	// Open File (on error, return showing error)
	if (!(myFile->SetTo(entry, B_READ_ONLY) == B_OK))
	{
		char name[B_FILE_NAME_LENGTH];
		
		entry->GetName(name);
		printf("File not found...  (%s)\n",name); 
		return false;
	}

	return true;
}

void RetrieveFromFile(BFile* myFile, BEntry* entry)
{
	ssize_t amt_read; 
	id3struct id3tag = {"\0","\0","\0","\0","\0","\0",155};
	char buf[128] = "\0";

	// Read from file into buf
	myFile->Seek(-128, SEEK_END);
	amt_read = myFile->Read((void *)buf, 128);

	// If first part of buf = "TAG" (it has a tag), split buf up into id3 tag parts
	if (strncmp(buf, "TAG", 3) == 0)
	{
		memcpy(id3tag.tag     , buf      , 3 );
		memcpy(id3tag.title   , buf + 3  , 30);
		memcpy(id3tag.artist  , buf + 33 , 30);
		memcpy(id3tag.album   , buf + 63 , 30);
		memcpy(id3tag.year    , buf + 93 , 4 );
		memcpy(id3tag.comment , buf + 97 , 30);
		       id3tag.genre   = buf[127];
	}

	BNode node(entry);
	node.WriteAttr("Audio:Artist",B_STRING_TYPE, 0, id3tag.artist,strlen(id3tag.artist)+1);
	node.WriteAttr("Audio:Title",B_STRING_TYPE, 0, id3tag.title,strlen(id3tag.title)+1);
	node.WriteAttr("Audio:Album",B_STRING_TYPE, 0, id3tag.album,strlen(id3tag.album)+1);
	
	int32 year = atoi(id3tag.year);
	
	node.WriteAttr("Audio:Year",B_INT32_TYPE, 0, (void *) &year, sizeof(int32) );
	node.WriteAttr("Audio:Comment",B_STRING_TYPE, 0, id3tag.comment,strlen(id3tag.comment)+1);
	//The next line probally doesn't work
	node.WriteAttr("Audio:GenreID",B_UINT32_TYPE, 0, (void *)id3tag.genre, 1);
	if (id3tag.genre >= 0 && id3tag.genre < genre_count)
		node.WriteAttr("Audio:Genre",B_STRING_TYPE, 0, genre_table[id3tag.genre],strlen(genre_table[id3tag.genre])+1);
}

int main( int argc, char **argv)
{
	int firstfile = 1;
	
	if (argc == 1) {
		printf("Usage: %s [-sr] <list of filenames and/or directories> \n", argv[0]);
		printf("   -r        recurse subdirectories\n");
		printf("   -s        Create x-mpeg mime types (if needed) and register id3 attributes \n             with the tracker\n");
		return 1;
	}
	
	if (strncmp(argv[1],"-s",2)==0) {
		
		//First Set up the system mime type
		BMimeType mime("audio/x-mpeg");
		if (mime.InitCheck() == B_OK)
			if (!mime.IsInstalled()) 
			{ 
				mime.Install();
			}
			
		//Tell the tracker about these great attributes
		BMessage msg, msg2;

  		msg.AddString("attr:name", "Audio:Artist"); 
  		msg.AddString("attr:public_name", "Artist"); 
  		msg.AddInt32("attr:type", B_STRING_TYPE); 
  		msg.AddBool("attr:viewable", true); 
  		msg.AddInt32("attr:alignment", B_ALIGN_LEFT); 

		msg.AddString("attr:name", "Audio:Title"); 
  		msg.AddString("attr:public_name", "Title"); 
  		msg.AddInt32("attr:type", B_STRING_TYPE); 
  		msg.AddBool("attr:viewable", true); 
  		msg.AddInt32("attr:alignment", B_ALIGN_LEFT); 
  		
  		msg.AddString("attr:name", "Audio:Album"); 
  		msg.AddString("attr:public_name", "Album"); 
  		msg.AddInt32("attr:type", B_STRING_TYPE); 
  		msg.AddBool("attr:viewable", true); 
  		msg.AddInt32("attr:alignment", B_ALIGN_LEFT); 

  		msg.AddString("attr:name", "Audio:Track"); 
  		msg.AddString("attr:public_name", "Track"); 
  		msg.AddInt32("attr:type", B_INT32_TYPE); 
  		msg.AddBool("attr:viewable", true); 
  		msg.AddInt32("attr:alignment", B_ALIGN_LEFT); 

  		msg.AddString("attr:name", "Audio:Year"); 
  		msg.AddString("attr:public_name", "Year"); 
  		msg.AddInt32("attr:type", B_INT32_TYPE); 
  		msg.AddBool("attr:viewable", true); 
  		msg.AddInt32("attr:alignment", B_ALIGN_LEFT); 

  		msg.AddString("attr:name", "Audio:Comment"); 
  		msg.AddString("attr:public_name", "Comment"); 
  		msg.AddInt32("attr:type", B_STRING_TYPE); 
  		msg.AddBool("attr:viewable", true); 
  		msg.AddInt32("attr:alignment", B_ALIGN_LEFT);
  		
  		msg.AddString("attr:name", "Audio:Genre"); 
  		msg.AddString("attr:public_name", "Genre"); 
  		msg.AddInt32("attr:type", B_STRING_TYPE); 
  		msg.AddBool("attr:viewable", true); 
  		msg.AddInt32("attr:alignment", B_ALIGN_LEFT);
  		
  		msg.AddString("attr:name", "Audio:Rating"); 
  		msg.AddString("attr:public_name", "Rating"); 
  		msg.AddInt32("attr:type", B_INT32_TYPE); 
  		msg.AddBool("attr:viewable", true); 
  		msg.AddInt32("attr:alignment", B_ALIGN_LEFT); 


		mime.SetAttrInfo(&msg);
		
		// Replace the attributes (TODO: Only add these as needed/ untrivial? )
		msg2.AddString("extensions","mp3");
		msg2.AddString("extensions","MP3");
		msg2.AddString("extensions","mp2");
		msg2.AddString("extensions","MP2");
		msg2.AddString("extensions","Mp3");
		msg2.AddString("extensions","mP3");
		msg2.AddString("extensions","Mp2");
		msg2.AddString("extensions","mP2");
		mime.SetFileExtensions(&msg2);
		
		// A nice little description
		mime.SetShortDescription("MPEG audio file");	
		
		//Set up indexies
		dev_t part;
		
		
		part  = dev_for_path(".");
		if (part >= 0) 
		{
			//printf("%d\n",part);
			if (fs_create_index(part,"Audio:Artist" , B_STRING_TYPE, 0)         != 0)   
				if (errno != B_FILE_EXISTS)
					printf("Index Creations Failed Audio:Artist (%s) \n",strerror(errno));

			if (fs_create_index(part,"Audio:Title" , B_STRING_TYPE, 0) != 0)
				if (errno != B_FILE_EXISTS)
					printf("Index Creations Failed Audio:Title (%s) \n",strerror(errno));
						 
			if  (fs_create_index(part,"Audio:Album" , B_STRING_TYPE, 0)        != 0)  
				if (errno != B_FILE_EXISTS)
					printf("Index Creations Failed Audio:Album (%s) \n",strerror(errno));
					
			if (fs_create_index(part,"Audio:Track" , B_INT32_TYPE, 0)        != 0)  
				if (errno != B_FILE_EXISTS)
					printf("Index Creations Failed Audio:Track (%s) \n",strerror(errno));

		    if (fs_create_index(part,"Audio:Year" , B_INT32_TYPE, 0)			  != 0) 
				if (errno != B_FILE_EXISTS)
					printf("Index Creations Failed Audio:Year (%s) \n",strerror(errno));

			if (fs_create_index(part,"Audio:Comment" , B_STRING_TYPE, 0)   != 0)
				if (errno != B_FILE_EXISTS)
					printf("Index Creations Failed Audio:Comment (%s) \n",strerror(errno));
					
		    if (fs_create_index(part,"Audio:Genre" , B_STRING_TYPE, 0)		  != 0)  
				if (errno != B_FILE_EXISTS)
					printf("Index Creations Failed Audio:Genre (%s) \n",strerror(errno));

		    if (fs_create_index(part,"Audio:Rating" , B_INT32_TYPE, 0)			  != 0) 
				if (errno != B_FILE_EXISTS)
					printf("Index Creations Failed Audio:Rating (%s) \n",strerror(errno));			
		}
		return 0;	
		
	}
	
	
	// set up directory following
	if (strncmp(argv[1],"-r",2) == 0) recursive = true, firstfile++;

	for (int foo=firstfile; (argc>foo); foo++) 
	{
		BEntry entry(argv[foo]);
		BFile myFile;
		if ((entry.InitCheck() == B_OK) && (OpenFile(&entry,&myFile))) 
			RetrieveFromFile(&myFile, &entry);
	}
	return 0; 
}
